# coding: utf-8

"""
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from nethsm.client.paths.health_state.get.operation import ApiForGet


class HealthState(
    ApiForGet,
):
    pass
