# coding: utf-8

"""
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from nethsm.client.shared_imports.header_imports import *  # pyright: ignore [reportWildcardImportFromLibrary]

from .content.application_octet_stream import schema as application_octet_stream_schema


class RequestBody(api_client.RequestBody):


    class ApplicationOctetStreamMediaType(api_client.MediaType):
        schema: typing_extensions.TypeAlias = application_octet_stream_schema.Schema
    content = {
        'application/octet-stream': ApplicationOctetStreamMediaType,
    }
    required = True
