# coding: utf-8

"""
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from nethsm.client.paths.config_tls_csr_pem.post.operation import ApiForPost


class ConfigTlsCsrPem(
    ApiForPost,
):
    pass
