# coding: utf-8

"""
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from nethsm.client import security_schemes

security_requirement_object: security_schemes.SecurityRequirementObject = {
    "basic": (),
}
