# coding: utf-8

"""
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from nethsm.client.paths.keys_key_id_restrictions_tags_tag.delete.operation import ApiForDelete
from nethsm.client.paths.keys_key_id_restrictions_tags_tag.put.operation import ApiForPut


class KeysKeyIDRestrictionsTagsTag(
    ApiForDelete,
    ApiForPut,
):
    pass
