# coding: utf-8

"""
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from nethsm.client.shared_imports.security_scheme_imports import *  # pyright: ignore [reportWildcardImportFromLibrary]

@dataclasses.dataclass
class Basic(security_schemes.HTTPBasicSecurityScheme):
    '''
    NetHSM supports Basic Authentication.

    '''
