# coding: utf-8

"""
    NetHSM
    All endpoints expect exactly the specified JSON. Additional properties will cause a Bad Request Error (400). All HTTP errors contain a JSON structure with an explanation of type string. All [base64](https://tools.ietf.org/html/rfc4648#section-4) encoded values are Big Endian.   # noqa: E501
    The version of the OpenAPI document: v1
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from __future__ import annotations
from nethsm.client.shared_imports.schema_imports import *  # pyright: ignore [reportWildcardImportFromLibrary]


from nethsm.client.components.schema import user_item


class UserListTuple(
    typing.Tuple[
        user_item.UserItemDict,
        ...
    ]
):

    def __new__(cls, arg: typing.Union[UserListTupleInput, UserListTuple], configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None):
        return UserList.validate(arg, configuration=configuration)
UserListTupleInput = typing.Union[
    typing.List[
        typing.Union[
            user_item.UserItemDictInput,
            user_item.UserItemDict,
        ],
    ],
    typing.Tuple[
        typing.Union[
            user_item.UserItemDictInput,
            user_item.UserItemDict,
        ],
        ...
    ]
]


@dataclasses.dataclass(frozen=True)
class UserList(
    schemas.Schema[schemas.immutabledict, UserListTuple]
):
    """NOTE: This class is auto generated by OpenAPI JSON Schema Generator.
    Ref: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator

    Do not edit the class manually.
    """
    types: typing.FrozenSet[typing.Type] = frozenset({tuple})
    items: typing.Type[user_item.UserItem] = dataclasses.field(default_factory=lambda: user_item.UserItem) # type: ignore
    type_to_output_cls: typing.Mapping[
        typing.Type,
        typing.Type
    ] = dataclasses.field(
        default_factory=lambda: {
            tuple: UserListTuple
        }
    )

    @classmethod
    def validate(
        cls,
        arg: typing.Union[
            UserListTupleInput,
            UserListTuple,
        ],
        configuration: typing.Optional[schema_configuration.SchemaConfiguration] = None
    ) -> UserListTuple:
        return super().validate_base(
            arg,
            configuration=configuration,
        )
