# coding: utf-8

"""
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from nethsm.client.paths.config_network.get.operation import ApiForGet
from nethsm.client.paths.config_network.put.operation import ApiForPut


class ConfigNetwork(
    ApiForGet,
    ApiForPut,
):
    pass
