# coding: utf-8

"""
    Generated by: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator
"""

from nethsm.client.paths.keys_key_id_csr_pem.post.operation import KeysKeyIDCsrPemPost
from nethsm.client.paths.system_factory_reset.post.operation import SystemFactoryResetPost
from nethsm.client.paths.system_commit_update.post.operation import SystemCommitUpdatePost
from nethsm.client.paths.system_update.post.operation import SystemUpdatePost
from nethsm.client.paths.system_info.get.operation import SystemInfoGet
from nethsm.client.paths.config_tls_csr_pem.post.operation import ConfigTlsCsrPemPost
from nethsm.client.paths.config_tls_generate.post.operation import ConfigTlsGeneratePost
from nethsm.client.paths.keys_key_id_decrypt.post.operation import KeysKeyIDDecryptPost
from nethsm.client.paths.users_user_id_tags.get.operation import UsersUserIDTagsGet
from nethsm.client.paths.config_tls_cert_pem.get.operation import ConfigTlsCertPemGet
from nethsm.client.paths.config_tls_cert_pem.put.operation import ConfigTlsCertPemPut
from nethsm.client.paths.system_backup.post.operation import SystemBackupPost
from nethsm.client.paths.provision.post.operation import ProvisionPost
from nethsm.client.paths.keys_key_id_sign.post.operation import KeysKeyIDSignPost
from nethsm.client.paths.health_state.get.operation import HealthStateGet
from nethsm.client.paths.keys_key_id_cert.delete.operation import KeysKeyIDCertDelete
from nethsm.client.paths.keys_key_id_cert.get.operation import KeysKeyIDCertGet
from nethsm.client.paths.keys_key_id_cert.put.operation import KeysKeyIDCertPut
from nethsm.client.paths.keys_key_id_encrypt.post.operation import KeysKeyIDEncryptPost
from nethsm.client.paths.system_cancel_update.post.operation import SystemCancelUpdatePost
from nethsm.client.paths.keys_key_id_public_pem.get.operation import KeysKeyIDPublicPemGet
from nethsm.client.paths.config_unlock_passphrase.put.operation import ConfigUnlockPassphrasePut
from nethsm.client.paths.system_reboot.post.operation import SystemRebootPost
from nethsm.client.paths.keys_key_id_restrictions_tags_tag.delete.operation import KeysKeyIDRestrictionsTagsTagDelete
from nethsm.client.paths.keys_key_id_restrictions_tags_tag.put.operation import KeysKeyIDRestrictionsTagsTagPut
from nethsm.client.paths.unlock.post.operation import UnlockPost
from nethsm.client.paths.config_logging.get.operation import ConfigLoggingGet
from nethsm.client.paths.config_logging.put.operation import ConfigLoggingPut
from nethsm.client.paths.lock.post.operation import LockPost
from nethsm.client.paths.config_unattended_boot.get.operation import ConfigUnattendedBootGet
from nethsm.client.paths.config_unattended_boot.put.operation import ConfigUnattendedBootPut
from nethsm.client.paths.keys_generate.post.operation import KeysGeneratePost
from nethsm.client.paths.config_tls_public_pem.get.operation import ConfigTlsPublicPemGet
from nethsm.client.paths.info.get.operation import InfoGet
from nethsm.client.paths.config_backup_passphrase.put.operation import ConfigBackupPassphrasePut
from nethsm.client.paths.health_ready.get.operation import HealthReadyGet
from nethsm.client.paths.config_time.get.operation import ConfigTimeGet
from nethsm.client.paths.config_time.put.operation import ConfigTimePut
from nethsm.client.paths.system_shutdown.post.operation import SystemShutdownPost
from nethsm.client.paths.config_network.get.operation import ConfigNetworkGet
from nethsm.client.paths.config_network.put.operation import ConfigNetworkPut
from nethsm.client.paths.keys_key_id.delete.operation import KeysKeyIDDelete
from nethsm.client.paths.keys_key_id.get.operation import KeysKeyIDGet
from nethsm.client.paths.keys_key_id.put.operation import KeysKeyIDPut
from nethsm.client.paths.health_alive.get.operation import HealthAliveGet
from nethsm.client.paths.metrics.get.operation import MetricsGet
from nethsm.client.paths.users_user_id_passphrase.post.operation import UsersUserIDPassphrasePost
from nethsm.client.paths.users_user_id.delete.operation import UsersUserIDDelete
from nethsm.client.paths.users_user_id.get.operation import UsersUserIDGet
from nethsm.client.paths.users_user_id.put.operation import UsersUserIDPut
from nethsm.client.paths.system_restore.post.operation import SystemRestorePost
from nethsm.client.paths.users_user_id_tags_tag.delete.operation import UsersUserIDTagsTagDelete
from nethsm.client.paths.users_user_id_tags_tag.put.operation import UsersUserIDTagsTagPut
from nethsm.client.paths.keys.get.operation import KeysGet
from nethsm.client.paths.keys.post.operation import KeysPost
from nethsm.client.paths.random.post.operation import RandomPost
from nethsm.client.paths.users.get.operation import UsersGet
from nethsm.client.paths.users.post.operation import UsersPost


class DefaultApi(
    KeysKeyIDCsrPemPost,
    SystemFactoryResetPost,
    SystemCommitUpdatePost,
    SystemUpdatePost,
    SystemInfoGet,
    ConfigTlsCsrPemPost,
    ConfigTlsGeneratePost,
    KeysKeyIDDecryptPost,
    UsersUserIDTagsGet,
    ConfigTlsCertPemGet,
    ConfigTlsCertPemPut,
    SystemBackupPost,
    ProvisionPost,
    KeysKeyIDSignPost,
    HealthStateGet,
    KeysKeyIDCertDelete,
    KeysKeyIDCertGet,
    KeysKeyIDCertPut,
    KeysKeyIDEncryptPost,
    SystemCancelUpdatePost,
    KeysKeyIDPublicPemGet,
    ConfigUnlockPassphrasePut,
    SystemRebootPost,
    KeysKeyIDRestrictionsTagsTagDelete,
    KeysKeyIDRestrictionsTagsTagPut,
    UnlockPost,
    ConfigLoggingGet,
    ConfigLoggingPut,
    LockPost,
    ConfigUnattendedBootGet,
    ConfigUnattendedBootPut,
    KeysGeneratePost,
    ConfigTlsPublicPemGet,
    InfoGet,
    ConfigBackupPassphrasePut,
    HealthReadyGet,
    ConfigTimeGet,
    ConfigTimePut,
    SystemShutdownPost,
    ConfigNetworkGet,
    ConfigNetworkPut,
    KeysKeyIDDelete,
    KeysKeyIDGet,
    KeysKeyIDPut,
    HealthAliveGet,
    MetricsGet,
    UsersUserIDPassphrasePost,
    UsersUserIDDelete,
    UsersUserIDGet,
    UsersUserIDPut,
    SystemRestorePost,
    UsersUserIDTagsTagDelete,
    UsersUserIDTagsTagPut,
    KeysGet,
    KeysPost,
    RandomPost,
    UsersGet,
    UsersPost,
):
    """NOTE: This class is auto generated by OpenAPI JSON Schema Generator
    Ref: https://github.com/openapi-json-schema-tools/openapi-json-schema-generator

    Do not edit the class manually.
    """
    pass
